local modname = ...
local M = {}
_G[modname] = M

local debug = false
local numberOfModules
local numberOfColumns
local slaveSelectPin
local columns = {}

local MAX7219_REG_DECODEMODE = 0x09
local MAX7219_REG_INTENSITY = 0x0A
local MAX7219_REG_SCANLIMIT = 0x0B
local MAX7219_REG_SHUTDOWN = 0x0C
local MAX7219_REG_DISPLAYTEST = 0x0F

local function sendByte(module, register, data)
    local trans = {}
    local spiRegister = {}
    local spiData = {}
    for i = 1, numberOfModules do
        spiRegister[i] = 0
        spiData[i] = 0
    end
    spiRegister[module] = register
    spiData[module] = data
    for i = 1, numberOfModules do
        device:transfer(string.char(spiRegister[i], spiData[i]))
    end
end

local function commit()
    for i = 1, numberOfColumns do
        local module = math.floor(((i - 1) / 8) + 1)
        local register = math.floor(((i - 1) % 8) + 1)
        sendByte(module, register, columns[i])
    end
end

function M.setup(config)
    local config = config or {debug = true, numberOfModules = 1, slaveSelectPin = 22, intensity = 6}

    numberOfModules = 1
    numberOfColumns = numberOfModules * 8
  
    busmaster = spi.master(spi.HSPI, {sclk = 19, mosi = 23, miso = 25})
    device_config = {cs = 22, mode = 0, freq = 1000000}
    device = busmaster:device(device_config)
   
    for i = 1, numberOfModules do
       sendByte(i, MAX7219_REG_SCANLIMIT, 7)
       sendByte(i, MAX7219_REG_DECODEMODE, 0)
       sendByte(i, MAX7219_REG_DISPLAYTEST, 0)
       sendByte(i, MAX7219_REG_INTENSITY, config.intensity and config.intensity or 1)
       sendByte(i, MAX7219_REG_SHUTDOWN, 1)
    end
end

function M.clear()
    for i = 1, numberOfColumns do
        columns[i] = 0
    end
    commit()
end

function M.write(chars, transformation)
    local transformation = transformation or {}
    local revByte
    
    if transformation.invert == true then
        revByte = require("reverseBytes")
    end
    
    local c = {}
    for i = 1, #chars do
        local char = chars[i]
        
        if transformation.rotate ~= nil then
            char = rotate(char, transformation.rotate == "left")
        end
        
        for k, v in ipairs(char) do
            if transformation.invert == true then
                local index = ((i - 1) * 8) + 8 - k + 1
                c[index] = revByte.GetReverseByte(v)
            else
                table.insert(c, v)
            end
        end
    end
    
    if (revByte ~= nil) then
        package.loaded[revByte] = nil
        _G[revByte] = nil
        revByte = nil
    end
    
    columns = c
    commit()
end

function M.setIntensity(intensity)
    for i = 1, numberOfModules do
        sendByte(i, MAX7219_REG_INTENSITY, intensity)
    end
end

function M.shutdown(shutdown)
    local shutdownReg = shutdown and 0 or 1
    
    for i = 1, numberOfModules do
        sendByte(i, MAX7219_REG_SHUTDOWN, shutdownReg)
    end
end

function M.write7segment(text, rAlign)
    local tab = {}
    local lenNoDots = text:gsub("%.", ""):len()
    if (lenNoDots < (8 * numberOfModules)) then
        if (rAlign) then
            text = string.rep(" ", (8 * numberOfModules) - lenNoDots) .. text
        else
            text = text .. string.rep(" ", (8 * numberOfModules) - lenNoDots)
        end
    end
    
    local wasdot = false
    font7seg = require("_font7seg")
    
    for i = string.len(text), 1, -1 do
        local currentChar = text:sub(i, i)
        
        if (currentChar == ".") then
            wasdot = true
        else
            if (wasdot) then
                wasdot = false
                table.insert(tab, font7seg.GetChar(currentChar) + 0x80)
            else
                table.insert(tab, font7seg.GetChar(currentChar))
            end
        end
    end
    
    package.loaded['font7seg'] = nil
    font7seg = nil
    max7219.write({tab}, {invert = false})
end

return M
